﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using C1.Win.Chart.Designer;
using C1.Win.Chart;

using WealthHealth.Model;

namespace WealthHealth
{
    public partial class Form1 : Form
    {
        WealthHealth.Services.DataService dataService;

        int year = 1800;
        Font yearFont;
        SolidBrush yearBrush;
        Dictionary<string, SolidBrush> dataBrushes = new Dictionary<string, SolidBrush>();
        Dictionary<string, SolidBrush> selDataBrushes = new Dictionary<string, SolidBrush>();
        Dictionary<string, SolidBrush> opaqDataBrushes = new Dictionary<string, SolidBrush>();

        Timer timer;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            
            dataService = new WealthHealth.Services.DataService();

            // year watermark
            yearFont = new Font(chart.Font.FontFamily, 120);
            yearBrush = new SolidBrush(Color.FromArgb(0x2000916f));

            // custom brushes for data
            dataBrushes.Add("Sub-Saharan Africa", new SolidBrush(Color.FromArgb(0x7F1F77B4)));
            dataBrushes.Add("South Asia", new SolidBrush(Color.FromArgb(0x7FFF7F0E)));
            dataBrushes.Add("Middle East & North Africa", new SolidBrush(Color.FromArgb(0x7F2CA02C)));
            dataBrushes.Add("America", new SolidBrush(Color.FromArgb(0x7FD62728)));
            dataBrushes.Add("Europe & Central Asia", new SolidBrush(Color.FromArgb(0x7F9467BD)));
            dataBrushes.Add("East Asia & Pacific", new SolidBrush(Color.FromArgb(0x7F8C564B)));

            selDataBrushes.Add("Sub-Saharan Africa", new SolidBrush(Color.FromArgb(0x2F1F77B4)));
            selDataBrushes.Add("South Asia", new SolidBrush(Color.FromArgb(0x2FFF7F0E)));
            selDataBrushes.Add("Middle East & North Africa", new SolidBrush(Color.FromArgb(0x2F2CA02C)));
            selDataBrushes.Add("America", new SolidBrush(Color.FromArgb(0x2FD62728)));
            selDataBrushes.Add("Europe & Central Asia", new SolidBrush(Color.FromArgb(0x2F9467BD)));
            selDataBrushes.Add("East Asia & Pacific", new SolidBrush(Color.FromArgb(0x2F8C564B)));

            opaqDataBrushes.Add("Sub-Saharan Africa", new SolidBrush(Color.FromArgb(0xFF, 0x1F, 0x77, 0xB4)));
            opaqDataBrushes.Add("South Asia", new SolidBrush(Color.FromArgb(0xFF, 0xFF, 0x7F, 0x0E)));
            opaqDataBrushes.Add("Middle East & North Africa", new SolidBrush(Color.FromArgb(0xFF, 0x2C, 0xA0, 0x2C)));
            opaqDataBrushes.Add("America", new SolidBrush(Color.FromArgb(0xFF, 0xD6, 0x27, 0x28)));
            opaqDataBrushes.Add("Europe & Central Asia", new SolidBrush(Color.FromArgb(0xFF, 0x94, 0x67, 0xBD)));
            opaqDataBrushes.Add("East Asia & Pacific", new SolidBrush(Color.FromArgb(0xFF, 0x8C, 0x56, 0x4B)));

            // timer for animation
            timer = new Timer() { Interval = 50 };
            timer.Tick += Timer_Tick;

            UpdateTrackingText();

            //Setup chart
            chart.BeginUpdate();
            chart.Series.Clear();

            //Use chart's binding properties
            chart.Binding = "LifeExpectancy,Population";
            chart.BindingX = "Income";

            //Add series to the chart
            var ser = new C1.Win.Chart.Series();
            ser.SymbolStyle.Stroke = Brushes.Transparent;
            ser.SymbolRendering += Ser_SymbolRendering;
            chart.Series.Add(ser);

            //Set up a chart type
            chart.ChartType = C1.Chart.ChartType.Bubble;

            //Invoke the chart designer 
            var designer = new ChartDesigner(chart);

            chart.Options.BubbleMaxSize = 100;

            chart.AxisX.LogBase = 10;
            chart.AxisX.Min = 300;
            chart.AxisX.Max = 100000;
            chart.AxisX.Title = "income per capita (inflation-adjusted US dollars)";

            chart.AxisY.Min = 20;
            chart.AxisY.Max = 85;
            chart.AxisY.Title = "life expectancy (years)";

            chart.Rendering += chart_Rendering;
            chart.ToolTip.Content = "{Country}\n{PopulationMillions}";

            chart.SelectionMode = C1.Chart.ChartSelectionMode.Point;
            chart.SelectionStyle.StrokeWidth = 3;
            chart.SelectionStyle.StrokeColor = Color.DarkBlue;

            chart.SelectionChanged += (s, a) => UpdateTrackingText();

            UpdateData();
            chart.EndUpdate();

        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            if (trackBar1.Value == trackBar1.Maximum)
                StopAnimation();
            else
                trackBar1.Value += 1;
        }

        private void chart_Rendering(object sender, C1.Win.Chart.RenderEventArgs e)
        {
            if (e.Graphics == null)
                return;

            // render year watermark
            var plotRect = chart.PlotRect;
            var s = year.ToString();
            var sz = e.Graphics.MeasureString(s, yearFont, new PointF(), StringFormat.GenericTypographic);
            e.Graphics.DrawString( s, yearFont, yearBrush, plotRect.Right - sz.Width, 
                plotRect.Bottom - sz.Height, StringFormat.GenericTypographic);
        }

        private void Ser_SymbolRendering(object sender, C1.Win.Chart.RenderSymbolEventArgs e)
        {
            var item = (DataItem)e.Item;
            // set symbol color based on country region
            if (chart.SelectedIndex >= 0)
            {
                if (chart.SelectedIndex == e.Index)
                    e.Engine.SetFill(dataBrushes[item.Region]);
                else
                    e.Engine.SetFill(selDataBrushes[item.Region]);
            }
            else
                e.Engine.SetFill(dataBrushes[item.Region]);
        }

        void UpdateData()
        {
            string selectedCountry = GetSelectedCountry();
            var data = dataService.GetDataByYear(year);
            chart.BeginUpdate();
            //Set a  datasource
            chart.DataSource = data;
            // index of country may be different 
            if (selectedCountry != null)
                chart.SelectedIndex = data.IndexOf(data.Find( item=>item.Country == selectedCountry));
            chart.EndUpdate();
        }

        void UpdateTrackingText()
        {
            textBox3.Text = chart.SelectedIndex >= 0 ? 
                string.Format("Now tracking: {0}", GetSelectedCountry()) :
                "Try clicking a bubble to track a country's evolution through the animation.";
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (timer.Enabled)
                StopAnimation();
            else
                StartAnimation();
        }

        private void trackBar1_ValueChanged(object sender, EventArgs e)
        {
            year = trackBar1.Value;
            UpdateData();
        }

        void StartAnimation()
        {
            timer.Start();
            button1.Text = "Stop";
        }

        void StopAnimation()
        {
            timer.Stop();
            button1.Text = "Start";
        }

        string GetSelectedCountry()
        {
            return chart.SelectedIndex >= 0 ? ((List<DataItem>)chart.DataSource)[chart.SelectedIndex].Country : null;
        }
    }
}
